import { Vector2, HazelReader, HazelWriter } from "@skeldjs/util";
import { BaseRpcMessage } from "@skeldjs/protocol";
import { SpawnType } from "@skeldjs/constant";
import { ExtractEventTypes } from "@skeldjs/events";
import { Networkable, NetworkableEvents } from "../../Networkable";
import { PlayerData } from "../../PlayerData";
import { Hostable } from "../../Hostable";
import { PlayerMoveEvent, PlayerSnapToEvent } from "../../events";
import { PlayerControl } from "../PlayerControl";
export interface CustomNetworkTransformData {
    seqId: number;
    position: Vector2;
    velocity: Vector2;
}
export declare type CustomNetworkTransformEvents<RoomType extends Hostable = Hostable> = NetworkableEvents<RoomType> & ExtractEventTypes<[
    PlayerMoveEvent<RoomType>,
    PlayerSnapToEvent<RoomType>
]>;
/**
 * Represents player component for networking movement.
 *
 * See {@link CustomNetworkTransformEvents} for events to listen to.
 */
export declare class CustomNetworkTransform<RoomType extends Hostable = Hostable> extends Networkable<CustomNetworkTransformData, CustomNetworkTransformEvents<RoomType>, RoomType> implements CustomNetworkTransformData {
    /**
     * The previous sequence ID.
     */
    oldSeqId: number;
    /**
     * The current sequence ID.
     */
    seqId: number;
    /**
     * The current position of the player.
     */
    position: Vector2;
    /**
     * The velocity of the player.
     */
    velocity: Vector2;
    /**
     * The player that this component belongs to.
     */
    player: PlayerData<RoomType>;
    constructor(room: RoomType, spawnType: SpawnType, netId: number, ownerid: number, flags: number, data?: HazelReader | CustomNetworkTransformData, playerControl?: PlayerControl<RoomType>);
    Deserialize(reader: HazelReader, spawn?: boolean): void;
    Serialize(writer: HazelWriter, spawn?: boolean): boolean;
    HandleRpc(rpc: BaseRpcMessage): Promise<void>;
    /**
     * Move to a position (lerps towards).
     * @param position The position to move towards.
     * @param velocity The velocity to display moving at.
     * @example
     *```typescript
     * // Follow the host
     * host.transform.on("player.move", ev => {
     *   player.transform.move(ev.position.x, ev.position.y);
     * });
     * ```
     */
    move(x: number, y: number, velocity?: Vector2): Promise<void>;
    private _handleSnapTo;
    private _snapTo;
    private _rpcSnapTo;
    /**
     * Instantly snap to a position without lerping.
     * @param position The position to snap to.
     * @example
     *```typescript
     * // Instantly teleport to wherever the host moves.
     * host.transform.on("player.move", ev => {
     *   client.me.transform.snapTo(ev.position);
     * });
     * ```
     */
    snapTo(position: Vector2, rpc: boolean | undefined): Promise<void>;
    /**
     * Instantly snap to a position without lerping.
     * @param x The X position to snap to.
     * @param y The Y position to snap to.
     * @example
     *```typescript
     * // Instantly teleport to wherever the host moves.
     * host.transform.on("player.move", ev => {
     *   client.me.transform.snapTo(ev.position.x, ev.position.y);
     * });
     * ```
     */
    snapTo(x: number, y: number, rpc: boolean | undefined): Promise<void>;
}
//# sourceMappingURL=CustomNetworkTransform.d.ts.map