"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ElectricalDoorsSystem = void 0;
const SystemStatus_1 = require("./SystemStatus");
const Door_1 = require("../misc/Door");
const events_1 = require("../events");
/**
 * Represents a system for doors that must be manually opened.
 *
 * See {@link ElectricalDoorsSystemEvents} for events to listen to.
 */
class ElectricalDoorsSystem extends SystemStatus_1.SystemStatus {
    constructor(ship, systemType, data) {
        super(ship, systemType, data);
        this.doors || (this.doors = []);
        this.doors = this.doors.map((door, i) => typeof door === "boolean"
            ? new Door_1.Door(this, i, door)
            : door);
    }
    /* eslint-disable-next-line @typescript-eslint/no-unused-vars */
    Deserialize(reader, spawn) {
        const dirtyBit = reader.uint32();
        for (let i = 0; i < this.doors.length; i++) {
            const isOpen = (dirtyBit & (1 << i)) > 0;
            if (isOpen) {
                this._openDoor(i, undefined, undefined);
            }
            else {
                this._closeDoor(i, undefined, undefined);
            }
        }
    }
    /* eslint-disable-next-line @typescript-eslint/no-unused-vars */
    Serialize(writer, spawn) {
        let dirtyBit = 0;
        for (let i = 0; i < this.doors.length; i++) {
            dirtyBit |= this.doors[i].isOpen << i;
        }
        writer.uint32(dirtyBit);
        this.dirty = spawn;
    }
    async _openDoor(doorId, player, rpc) {
        const door = this.doors[doorId];
        if (!door)
            return;
        door.isOpen = true;
        this.dirty = true;
        const ev = await door.emit(new events_1.DoorsDoorOpenEvent(this.room, this, rpc, player, door));
        if (ev.reverted) {
            door.isOpen = false;
            return;
        }
        if (ev.alteredDoor !== door) {
            door.isOpen = false;
            ev.alteredDoor.isOpen = true;
        }
    }
    /**
     * Open a door by its ID. This is a host operation on official servers.
     * @param doorId the ID of the door to open
     */
    async openDoor(doorId) {
        await this._openDoor(doorId, this.room.myPlayer, undefined);
    }
    async _closeDoor(doorId, player, rpc) {
        const door = this.doors[doorId];
        if (!door)
            return;
        door.isOpen = false;
        this.dirty = true;
        const ev = await door.emit(new events_1.DoorsDoorCloseEvent(this.room, this, rpc, player, door));
        if (ev.reverted) {
            door.isOpen = true;
            return;
        }
        if (ev.alteredDoor !== door) {
            door.isOpen = true;
            ev.alteredDoor.isOpen = false;
        }
    }
    /**
     * Close a door by its ID. This is a host operation on official servers.
     * @param doorId The ID of the door to close.
     */
    async closeDoor(doorId) {
        await this._closeDoor(doorId, this.room.myPlayer, undefined);
    }
}
exports.ElectricalDoorsSystem = ElectricalDoorsSystem;
//# sourceMappingURL=ElectricalDoorsSystem.js.map